@@ -1,4 +1,4 @@
-/* 26jan15abu
+/* 24nov16abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -742,6 +742,19 @@ any doPop(any ex) {
    return car(y);
 }
 
+// (++ var) -> any
+any doPopq(any ex) {
+   any x, y;
+
+   x = cadr(ex);
+   NeedVar(ex,x);
+   CheckVar(ex,x);
+   if (!isCell(y = val(x)))
+      return y;
+   val(x) = cdr(y);
+   return car(y);
+}
+
 // (cut 'cnt 'var) -> lst
 any doCut(any ex) {
    long n;
